@@ -251,6 +251,15 @@ textbox *textbox_create(widget *parent, WidgetType type, const char *name,
       tb->placeholder = g_markup_escape_text(placeholder, -1);
     }
   }
+
+  const char *password_mask_char =
+      rofi_theme_get_string(WIDGET(tb), "password-mask", NULL);
+  if (password_mask_char == NULL || (*password_mask_char) == '\0') {
+    tb->password_mask_char = "*";
+  } else {
+    tb->password_mask_char = password_mask_char;
+  }
+
   textbox_text(tb, txt ? txt : "");
   textbox_cursor_end(tb);
 
@@ -337,11 +346,14 @@ static void __textbox_update_pango_text(textbox *tb) {
   }
   tb->show_placeholder = FALSE;
   if ((tb->flags & TB_PASSWORD) == TB_PASSWORD) {
-    size_t l = g_utf8_strlen(tb->text, -1);
-    char string[l + 1];
-    memset(string, '*', l);
-    string[l] = '\0';
-    pango_layout_set_text(tb->layout, string, l);
+    size_t text_len = g_utf8_strlen(tb->text, -1);
+    size_t mask_len = strlen(tb->password_mask_char);
+    char string[text_len * mask_len + 1];
+    for (size_t offset = 0; offset < text_len * mask_len; offset += mask_len) {
+      memcpy(string + offset, tb->password_mask_char, mask_len);
+    }
+    string[text_len * mask_len] = '\0';
+    pango_layout_set_text(tb->layout, string, -1);
   } else if (tb->flags & TB_MARKUP || tb->tbft & MARKUP) {
     pango_layout_set_markup(tb->layout, tb->text, -1);
   } else {
@@ -489,7 +501,6 @@ static void textbox_draw(widget *wid, cairo_t *draw) {
     return;
   }
   textbox *tb = (textbox *)wid;
-  int dot_offset = 0;
 
   if (tb->changed) {
     __textbox_update_pango_text(tb);
@@ -522,7 +533,7 @@ static void textbox_draw(widget *wid, cairo_t *draw) {
   {
     int rem =
         MAX(0, tb->widget.w - widget_padding_get_padding_width(WIDGET(tb)) -
-                   line_width - dot_offset);
+                   line_width);
     switch (pango_layout_get_alignment(tb->layout)) {
     case PANGO_ALIGN_CENTER:
       x = rem * (tb->xalign - 0.5);
@@ -531,7 +542,7 @@ static void textbox_draw(widget *wid, cairo_t *draw) {
       x = rem * (tb->xalign - 1.0);
       break;
     default:
-      x = rem * tb->xalign + dot_offset;
+      x = rem * tb->xalign;
       break;
     }
     x += widget_padding_get_left(WIDGET(tb));
@@ -541,42 +552,54 @@ static void textbox_draw(widget *wid, cairo_t *draw) {
   if (tb->flags & TB_EDITABLE) {
     // We want to place the cursor based on the text shown.
     const char *text = pango_layout_get_text(tb->layout);
-    // Clamp the position, should not be needed, but we are paranoid.
-    int cursor_offset = MIN(tb->cursor, g_utf8_strlen(text, -1));
-    PangoRectangle pos;
-    // convert to byte location.
-    char *offset = g_utf8_offset_to_pointer(text, cursor_offset);
-    pango_layout_get_cursor_pos(tb->layout, offset - text, &pos, NULL);
-    int cursor_x = pos.x / PANGO_SCALE;
-    int cursor_y = pos.y / PANGO_SCALE;
-    int cursor_height = pos.height / PANGO_SCALE;
-    RofiDistance cursor_width =
-        rofi_theme_get_distance(WIDGET(tb), "cursor-width", 2);
-    int cursor_pixel_width =
-        distance_get_pixel(cursor_width, ROFI_ORIENTATION_HORIZONTAL);
-    if ((x + cursor_x) != tb->cursor_x_pos) {
-      tb->cursor_x_pos = x + cursor_x;
-    }
-    if (tb->blink) {
-      // This save/restore state is necessary to render the text in the
-      // correct color when `cursor-color` is set
-      cairo_save(draw);
-      // use text color as fallback for themes that don't specify the cursor
-      // color
-      rofi_theme_get_color(WIDGET(tb), "cursor-color", draw);
-      cairo_rectangle(draw, x + cursor_x, y + cursor_y, cursor_pixel_width,
-                      cursor_height);
-      if (rofi_theme_get_boolean(WIDGET(tb), "cursor-outline", FALSE)) {
-        cairo_fill_preserve(draw);
-        rofi_theme_get_color(WIDGET(tb), "cursor-outline-color", draw);
-        double width =
-            rofi_theme_get_double(WIDGET(tb), "cursor-outline-width", 0.5);
-        cairo_set_line_width(draw, width);
-        cairo_stroke(draw);
+    // hide the cursor, if no text is entered and hide-empty-cursor is set to true
+    if (!(tb->text[0] == '\0' && rofi_theme_get_boolean(WIDGET(tb), "hide-cursor-on-empty", FALSE) == TRUE)){
+      // Clamp the position, should not be needed, but we are paranoid.
+      size_t cursor_offset;
+
+      if ((tb->flags & TB_PASSWORD) == TB_PASSWORD) {
+        // Calculate cursor position based on mask length
+        size_t mask_len = strlen(tb->password_mask_char);
+        cursor_offset = MIN(tb->cursor * mask_len, strlen(text));
       } else {
-        cairo_fill(draw);
+        cursor_offset = MIN(tb->cursor, g_utf8_strlen(text, -1));
+        // convert to byte location.
+        char *offset = g_utf8_offset_to_pointer(text, cursor_offset);
+        cursor_offset = offset - text;
+      }
+      PangoRectangle pos;
+      pango_layout_get_cursor_pos(tb->layout, cursor_offset, &pos, NULL);
+      int cursor_x = pos.x / PANGO_SCALE;
+      int cursor_y = pos.y / PANGO_SCALE;
+      int cursor_height = pos.height / PANGO_SCALE;
+      RofiDistance cursor_width =
+          rofi_theme_get_distance(WIDGET(tb), "cursor-width", 2);
+      int cursor_pixel_width =
+          distance_get_pixel(cursor_width, ROFI_ORIENTATION_HORIZONTAL);
+      if ((x + cursor_x) != tb->cursor_x_pos) {
+        tb->cursor_x_pos = x + cursor_x;
+      }
+      if (tb->blink) {
+        // This save/restore state is necessary to render the text in the
+        // correct color when `cursor-color` is set
+        cairo_save(draw);
+        // use text color as fallback for themes that don't specify the cursor
+        // color
+        rofi_theme_get_color(WIDGET(tb), "cursor-color", draw);
+        cairo_rectangle(draw, x + cursor_x, y + cursor_y, cursor_pixel_width,
+                        cursor_height);
+        if (rofi_theme_get_boolean(WIDGET(tb), "cursor-outline", FALSE)) {
+          cairo_fill_preserve(draw);
+          rofi_theme_get_color(WIDGET(tb), "cursor-outline-color", draw);
+          double width =
+              rofi_theme_get_double(WIDGET(tb), "cursor-outline-width", 0.5);
+          cairo_set_line_width(draw, width);
+          cairo_stroke(draw);
+        } else {
+          cairo_fill(draw);
+        }
+        cairo_restore(draw);
       }
-      cairo_restore(draw);
     }
   }
 
