@@ -136,6 +136,43 @@ static XrmOption xrmOptions[] = {
      "Custom command to generate preview icons",
      CONFIG_DEFAULT},
 
+    {xrm_String,
+     "on-selection-changed",
+     {.str = &config.on_selection_changed},
+     NULL,
+     "Custom command to call when menu selection changes",
+     CONFIG_DEFAULT},
+    {xrm_String,
+     "on-mode-changed",
+     {.str = &config.on_mode_changed},
+     NULL,
+     "Custom command to call when menu mode changes",
+     CONFIG_DEFAULT},
+    {xrm_String,
+     "on-entry-accepted",
+     {.str = &config.on_entry_accepted},
+     NULL,
+     "Custom command to call when menu entry is accepted",
+     CONFIG_DEFAULT},
+    {xrm_String,
+     "on-menu-canceled",
+     {.str = &config.on_menu_canceled},
+     NULL,
+     "Custom command to call when menu is canceled",
+     CONFIG_DEFAULT},
+    {xrm_String,
+     "on-menu-error",
+     {.str = &config.on_menu_error},
+     NULL,
+     "Custom command to call when menu finds errors",
+     CONFIG_DEFAULT},
+    {xrm_String,
+     "on-screenshot-taken",
+     {.str = &config.on_screenshot_taken},
+     NULL,
+     "Custom command to call when menu screenshot is taken",
+     CONFIG_DEFAULT},
+
     {xrm_String,
      "terminal",
      {.str = &config.terminal_emulator},
@@ -204,6 +241,12 @@ static XrmOption xrmOptions[] = {
      NULL,
      "Only show Desktop entry from these categories",
      CONFIG_DEFAULT},
+    {xrm_String,
+    "drun-exclude-categories",
+    {.str = &config.drun_exclude_categories},
+    NULL,
+    "Exclude Desktop entries from these categories",
+    CONFIG_DEFAULT},
     {xrm_Boolean,
      "drun-show-actions",
      {.num = &config.drun_show_actions},
@@ -253,6 +296,12 @@ static XrmOption xrmOptions[] = {
      NULL,
      "Set case-sensitivity",
      CONFIG_DEFAULT},
+    {xrm_Boolean,
+     "case-smart",
+     {.num = &config.case_smart},
+     NULL,
+     "Set smartcase like vim (determine case-sensitivity by input)",
+     CONFIG_DEFAULT},
     {xrm_Boolean,
      "cycle",
      {.num = &config.cycle},
@@ -452,6 +501,12 @@ static XrmOption xrmOptions[] = {
      NULL,
      "What completer to use for drun/run.",
      CONFIG_DEFAULT},
+    {xrm_Boolean,
+     "imdkit",
+     {.snum = &config.enable_imdkit},
+     NULL,
+     "Whether to enable imdkit",
+     CONFIG_DEFAULT},
 };
 
 /** Dynamic array of extra options */
